<?php 
/**
 * Query tools for nodes
 */


/**
 * Based on the core node_load().
 * Load a node object from the database, using both core node fields and CCK fields.
 * Apapted from http://drupal.org/node/360541#comment-3669676
 *
 * @param $param
 *   An array of conditions for core node fields to match against in the database query.
 *   Since we also have the CCK conditions, this parameter can also be NULL.
 * @param $cck_param
 *   An array of conditions for CCK fields to match against in the database query
 * @param $revision
 *   Which numbered revision to load. Defaults to the current version.
 *
 * @return
 *   A fully-populated node object.
 *
 * usage:
 *  $node = query_tools_node_find(
 *    array('type' => 'product'),
 *    array('price' => '100')
 *  );
 */
function query_tools_node_find($parameters = NULL, $cck_param = array(), $revision = NULL) {
  $arguments = array();
  $rnode = array();

  /**
   * Process regular parameters
   */
  if (is_array($parameters)) {
    // Turn the conditions into a query.
    foreach ($parameters as $key => $value) {
      $cond[] = 'n.'. db_escape_table($key) ." = '%s'";
      $arguments[] = $value;
    }
    $cond = implode(' AND ', $cond);
  }

  /**
   * Process CCK parameters
   */
  $cck_tables = array();
  if (count($cck_param)) {
    // Turn the conditions into a query.
    $i = 1;
    foreach ($cck_param as $key => $value) {
      $field_name = 'field_'. str_replace('field_', '', $key); // ensure field has 'field_' at the beginning
      $db_info = content_database_info(content_fields($field_name));
      // If the field is from another table, increment sequence
      if(!isset($cck_tables[$db_info['table']])){
        $cck_tables[$db_info['table']] = 'cck'. $i;
      }
      $i++;
      // nodereference fields
      if($db_info['columns']['nid']){
        $cck_cond[] = $cck_tables[$db_info['table']] .'.'. db_escape_table($db_info['columns']['nid']['column']) ." = '%s'";
      } 
      // Normal fields
      elseif ($db_info['columns']['value']) {
        $cck_cond[] = $cck_tables[$db_info['table']] .'.'. db_escape_table($db_info['columns']['value']['column']) ." = '%s'";
      } 
      // User reference fields
      elseif ($db_info['columns']['uid']) {
        $cck_cond[] = $cck_tables[$db_info['table']] .'.'. db_escape_table($db_info['columns']['uid']['column']) ." = '%s'";
      }
      elseif ($db_info['columns'][$key]['column']) {
        $cck_cond[] = $cck_tables[$db_info['table']] .'.'. db_escape_table($db_info['columns'][$key]['column']) ." = '%s'";
      }
      $arguments[] = $value;
    }
    if (is_array($cck_cond)) {
      $cond .= (isset($cond) ? ' AND ' : ''). implode(' AND ', $cck_cond);
      foreach ($cck_tables as $table => $nick) {
        $cck_join .= ' INNER JOIN {'. $table .'} '. $nick .' ON '. $nick .'.nid = n.nid';
      }
    }
  }

  if ($revision) {
    array_unshift($arguments, $revision);
    $sql = 'SELECT n.nid FROM {node} n INNER JOIN {node_revisions} r ON r.vid = n.vid'. $cck_join .' AND r.vid = %d WHERE '. $cond;
    $nodes = db_query($sql, $arguments);
  }
  else {
    $sql = 'SELECT n.nid FROM {node} n INNER JOIN {node_revisions} r ON r.vid = n.vid'. $cck_join .' WHERE '. $cond;
    $nodes = db_query($sql, $arguments);
  }

  /**
   * Iterate over retrieved node values, and give node_load an opportunity 
   * to add more fields.
   */
  while ($node = db_fetch_object($nodes)) {
    if ($node && $node->nid) {
      $rnodes[] = node_load($node->nid);
    }
  }
  
  // RETURN
  return $rnodes;
}

/**
 * Update a node
 */
function query_tools_node_update($nid, $fields){
  
  watchdog('query_tools', 'Updating '.$nid);
  
  $node = node_load($nid);
  
  foreach($fields as $field){
    switch ($field['#type']){
      case 'cck':
        query_tools_set_field($node, $field);
        break;
      default:
        query_tools_set_property($node, $field);
        break;
    } 
  }
  
  $result = query_tools_node_save($node);
  
  return $result;
}

/**
 * Update a field value
 */
function query_tools_set_field(&$node, $field){
  $item = array();
  
  $item['field'] = $field;
  $item['field_info'] = content_fields($field['#field_name']);
  $item['db_info'] = content_database_info($item['field']);
  
  $node->$field['#field_name'] = $field['#values'];
  return TRUE;
}

/**
 * Update a default property
 */
function query_tools_set_property(&$node, $field){
  return TRUE;
}

/**
 * Intialise a node object with sensible defaults
 */
function _query_tools_init_node_object(){
  $node = new stdClass();

  //Main Node Fields
  $node->name = 'Node name';
  $node->title = 'Node title';
  $node->body = '';
  $node->type = 'page'; //This can be any node type
  $node->created = time();
  $node->changed = $node->created;
  $node->promote = 0; // Display on front page ? 1 : 0
  $node->sticky = 0;  // Display top of page ? 1 : 0
  $node->format = 2;  // 1:Filtered HTML, 2: Full HTML
  $node->status = 1;   // Published ? 1 : 0
  $node->language = 'en';
  
  return $node;
}

/**
 * Save the node
 */
function query_tools_node_save($node){

  if ($node = node_submit($node)) {
  	node_save($node);
  	watchdog('query_tools', t("Node ".$node->title." updated correctly"));
  	return TRUE;
  } else {
  	watchdog('query_tools', t("Node ".$node->title." updated incorrectly"), "error");
  	return FALSE;
  }
}
